// Part of DTSource. Copyright 2004-2012. David Adalsteinsson.  BSD License
// see http://www.visualdatatools.com/DTSource/license.html for more information.

#ifndef DTIndex_Header
#define DTIndex_Header

// Index class to support array subscripts.

// Usage:
// A(3,DTIndex::All) = 3;   - Overwrites row 3.
// A(DTIndex::Last,DTIndex(0,2,DTIndex::Last)) = 5; - Overwrites even columns in last row.


#include "DTIntArray.h"

class DTIndex {
public:
    DTIndex();
    DTIndex(int number);
    DTIndex(int start,int end);
    DTIndex(int start,int stride,int end);
    DTIndex(const DTIntArray &specified);

    static int Last;
    static DTIndex All;

    // Need to supply the length before calling start,stride,end.
    bool SetLength(int howLong); // Returns false if invalid.

    bool IsEverything(void) const {return (stride==1 && start==0 && end==knownLength-1);}
    int Length(void) const;
    
    bool IsEmpty(void) const;
    bool ArraySpecified(void) const {return arraySpecified;}
    DTIntArray IndicesAsArray(void) const; // Will create it if necessary.
    DTIntArray SpecifiedArray(void) const {return specified;}
    int Start(void) const;
    int Stride(void) const;
    int End(void) const;
    
private:
    int knownLength;
    int start,stride,end;
    bool arraySpecified;
    DTIntArray specified;
};

extern DTIndex operator-(const DTIndex &,int);
extern DTIndex operator+(const DTIndex &,int);

#endif
