// Part of DTSource. Copyright 2004-2012. David Adalsteinsson.  BSD License
// see http://www.visualdatatools.com/DTSource/license.html for more information.

#include "DTArguments.h"

#include "DTError.h"

#include <vector>
#include <algorithm>

static vector<string> DTInputArgumentList;
static string DTApplicationName;

void DTSetArguments(int argc,const char *argv[])
{
    // Just copy the arguments into a list of strings.
    DTInputArgumentList.clear();

    DTApplicationName = argv[0];

    for (int i=1;i<argc;i++)
        DTInputArgumentList.push_back(string(argv[i]));
}

bool DTArgumentIncludesFlag(string flagName)
{
    string checkFor = "-"+flagName;

    vector<string>::const_iterator result;
    result = find(DTInputArgumentList.begin(),
                  DTInputArgumentList.end(),
                  checkFor);

    return (result!=DTInputArgumentList.end());
}

string DTArgumentFlagEntry(string flagName)
{
    string checkFor = "-"+flagName;

    vector<string>::const_iterator result;
    result = find(DTInputArgumentList.begin(),
                  DTInputArgumentList.end(),
                  checkFor);

    if (result==DTInputArgumentList.end())
        return string();

    ++result;
    string toReturn = *result;
    if (toReturn.length()==0 || toReturn[0]=='-')
        return string();

    return toReturn;
}

int DTArgumentIntegerEntry(string flagName)
{
    string strVal = DTArgumentFlagEntry(flagName);
    return atoi(strVal.c_str());
}

int DTHowManyArguments(void)
{
    return DTInputArgumentList.size();
}

string DTGetArgumentNumber(int num)
{
    if (num<0 || num>=int(DTInputArgumentList.size())) {
        DTErrorMessage("DTGetArgumentNumber(number)","number out of bounds.");
        return string();
    }

    return DTInputArgumentList[num];
}

