//
//  DrawRoutine.m
//  Speed Test
//
//  Created by David Adalsteinsson on 9/29/12.
//  Copyright (c) 2012 David Adalsteinsson. All rights reserved.
//

#import "DrawRoutine.h"

#import "DTDataFile.h"
#import "DTRegion1D.h"

@implementation DrawRoutine

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
    }
    
    return self;
}

- (void)drawRect:(NSRect)dirtyRect
{
    if (plot==NULL) {
        // Need to load this
        NSString *where = [[NSBundle mainBundle] pathForResource:@"dataFile" ofType:@"dtbin"];
        DTDataFile dataFile([where fileSystemRepresentation],DTFile::ExistingReadWrite);
        plot = new DTPlot1D();
        Read(dataFile,"path",*plot);
    }
    if (plot==NULL) return;

    // Figure out the scaling
    DTRegion1D domain = BoundingRange(*plot);
    DTRegion1D value = ValueRange(*plot);
    
    NSSize size = [self frame].size;
    double xshift = domain.minV;
    double xscale = size.width/(domain.maxV-domain.minV);
    double yshift = value.minV;
    double yscale = size.height/(value.maxV-value.minV);
    
    // Create the vertex list
    DTDoubleArray rawList = plot->Data();
    long int i;
    long int howMany = int(rawList(1,0)); // How the plot is packed.
    DTMutableDoubleArray vertices(2,howMany);
    for (i=0;i<howMany;i++) {
        vertices(0,i) = xscale*(rawList(0,i+1)-xshift);
        vertices(1,i) = yscale*(rawList(1,i+1)-yshift);
    }
    
    // Prepare the drawing context
    CGContextRef ctx = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];
    // Set line width and color
    CGFloat backingScaleFactor = [[self window] backingScaleFactor];
    CGFloat lineWidth = (thickLine ? 1.00001 : 1.0)/backingScaleFactor; // Anything that causes more than a pixel on the screen.
    CGColorRef cgColor = CGColorCreateGenericGray(0.5,1.0);
    CGContextSetStrokeColorWithColor(ctx,cgColor);
    CGContextSetLineWidth((CGContextRef)ctx,lineWidth);
    // The line join will affect the speed, but all are slow for the thick line
    CGContextSetLineJoin(ctx,kCGLineJoinBevel); // kCGLineJoinMiter, kCGLineJoinRound, kCGLineJoinBevel
    
    // Draw the path
    CGPoint *vertexList = (CGPoint *)vertices.Pointer();
    CGContextBeginPath(ctx);
    CGContextAddLines(ctx,vertexList,howMany);
    CGContextStrokePath(ctx);

    // Clean up
    CGColorRelease(cgColor);
}

- (void)toggleThickness:(id)sender
{
    thickLine = ([sender state]==1);
    [self setNeedsDisplay:YES];
}

@end
